%% Active Contraction
% Roxanna Barry
% University of Glasgow
%
% Numerical Solution of Discrete ODEs and Continuum PDEs 
% for single line of Neo-Hookean incompressible cells, 
% including active contraction and substrate damping.

function ActiveContraction


%% System Parameters

%
% % Discrete System
% 

% Number of cells
N = 100;
% Shear modulus
mu = ones(1,N);
% Resting/initial length of cells
L0 = ones(1,N);
% Active Contraction period
omega1=1/N;
% Resting/initial width of cells
W0=1;
% Non-dimensional Stokes Damping Coefficient
param = 50;
Ks = linspace(1e-5,10,param);
Ks(2) = 1e-4;
Ks(3) = 1e-3;
Ks(4) = 1e-2;
Ks(5) = 1e-1;
Ks(6) = 1;
% Non-dimensional length-width ratio
gam0 = W0/L0(1);

%
% % Continuum System
%

% Discretisation Points
nc = 50;
% Discretisation Step Size
dX = N/(nc-1);
% Resting length of cells
l0=1;
% Shear modulus
muhat = ones(1,nc);

%% Initial Conditions

% % Discrete System
x0=0:1:N;

% % Continuum System
X0 = 0:dX:N;

% Time span
tspan = 0:0.01:10;

% Initialise Storage Matrices
% Discrete
maxp = zeros(length(tspan),param);
minp = zeros(length(tspan),param);
minstretch = zeros(length(tspan),param);
maxstretch = zeros(length(tspan),param);
% Continuum
maxphat = zeros(length(tspan),param);
minphat = zeros(length(tspan),param);
minstretchhat = zeros(length(tspan),param);
maxstretchhat = zeros(length(tspan),param);

%% Loop over K values

for loop=1:param
    
    K = Ks(loop);

%% Solve the ODE/PDE System

% % Discrete System
m = eye(N+1,N+1);
a = 1e-10;
r = 1e-6;
options = odeset('Mass',m,'AbsTol',a,'RelTol',r);
[t,x] = ode15s(@NHBdry_ODE_AC, tspan, x0,options);
tend = length(t);

% % Continuum System
M = eye(nc);
A = 1e-12;
R = 1e-12;
Options = odeset('Mass',M,'AbsTol',A,'RelTol',R);
[T,X] = ode15s(@NHBdry_contODE_AC, tspan, X0,Options);
Tend = length(T);

%% Active Contraction

% % Discrete
omega = 2*pi*omega1;
active=zeros(tend,N);
tr = 2*pi*(x0)/N;
for it1=1:tend
    for it2 = 1:N
        active(it1,it2) = (1+(sin((omega*t(it1)-tr(it2)))));
    end
end

% % Continuum
omegahat = 2*pi*omega1;
activehat=zeros(Tend,nc);
trhat = 2*pi*X0/N;
for it3 = 1:nc
    activehat(:,it3) = (1+(sin((omegahat*T-trhat(it3)))));
end


%% Stretch and Pressure

%
% % Discrete System
% 

xpos = x;
for it4=1:N+1
    xpos(:,it4) = x(:,it4) - x(:,1);
end
x=xpos;

% Cell Centre of Mass
xcpos=zeros(tend,N);
for it4=1:N
    xcpos(:,it4) = (x(:,it4+1) + x(:,it4))/2;
end

% Stretches, Widths, Pressures and Stresses (and averages)
lambda = zeros(tend,N); 
W = zeros(tend,N); 
p = zeros(tend,N); 
stress = zeros(tend,N);
avep = zeros(tend,1);
avelambda = zeros(tend,1);
for it5 = 1:tend
    lambda(it5,:) = (x(it5,2:N+1)-x(it5,1:N))./L0(1:N); 
    W(it5,:) = 1./lambda(it5,:); 
    p(it5,:) = mu(1,1:N).*(lambda(it5,:).^(-2));
    avep(it5) = mean(p(it5,:));
    avelambda(it5) = mean(lambda(it5,:));
    stress(it5,:) = mu(1,1:N).*...
        (lambda(it5,:).^(2)-lambda(it5,:).^(-2));
end
W=gam0*W;

% Max and min values to save
minstretch(:,loop) = min(lambda,[],2);
maxstretch(:,loop) = max(lambda,[],2);
minp(:,loop) = min(p,[],2);
maxp(:,loop) = max(p,[],2);

%
% % Continuum System
%
Xpos = X;
for it6=1:nc
    Xpos(:,it6) = X(:,it6) - X(:,1);
end
X=Xpos;

% Stretches, Widths, Pressures and Stresses
lambdahat = zeros(Tend,nc); 
What = zeros(Tend,nc); 
phat = zeros(Tend,nc);
avephat = zeros(Tend,1);
avelambdahat = zeros(Tend,1);
stresshat = zeros(Tend,nc);
for it7=2:nc-1
    lambdahat(:,it7) = (X(:,it7+1) - X(:,it7-1))/2/dX; 
    What(:,it7) = lambdahat(:,it7).^(-1); 
end
lambdahat(:,1) = (X(:,2) - (X(:,nc-1)-N*l0(1)))/2/dX; 
What(:,1) = lambdahat(:,1).^(-1);
lambdahat(:,nc) = lambdahat(:,1); 
What(:,nc) = lambdahat(:,nc).^(-1);
for it8=1:Tend
    phat(it8,:) = muhat(1,1:nc).*(lambdahat(it8,:).^(-2)); 
    avelambdahat(it8) = mean(lambdahat(it8,:));
    avephat(it8) = mean(phat(it8,:));
    stresshat(it8,:) = muhat(1,1:nc).*...
        (lambdahat(it8,:).^(2)-lambdahat(it8,:).^(-2));
end
What=gam0*What;

figure(1)

plot(t,phat)

return

% Max and min values to save
minstretchhat(:,loop) = min(lambdahat,[],2);
maxstretchhat(:,loop) = max(lambdahat,[],2);
minphat(:,loop) = min(phat,[],2);
maxphat(:,loop) = max(phat,[],2);


% Save full simulation data for specific K values
if K == 1e-5 || K == 1e-3 || K == 1e-1 || K == 1 
    save(['K' num2str(K) '_D_AC_x.dat'],'x')
    save(['K' num2str(K) '_D_AC_xc.dat'],'xcpos')
    save(['K' num2str(K) '_C_AC_x.dat'],'X')
    save(['K' num2str(K) '_D_AC_p.dat'],'p')
    save(['K' num2str(K) '_C_AC_p.dat'],'phat')
    save(['K' num2str(K) '_D_AC_lambda.dat'],'lambda')
    save(['K' num2str(K) '_C_AC_lambda.dat'],'lambdahat')
    save(['K' num2str(K) '_D_AC_W.dat'],'W')
    save(['K' num2str(K) '_C_AC_W.dat'],'What')
    save(['K' num2str(K) '_D_AC_t.dat'],'t')
    save(['K' num2str(K) '_C_AC_t.dat'],'T')
end


% keep track of how many simulations have been completed & saved
disp(['Sim ' num2str(loop) ' of ' num2str(param) ' completed'])

end

% Save Parameter Data
save('K_AC_Ks.dat','Ks')
save('K_D_AC_MaxP.dat','maxp')
save('K_C_AC_MaxP.dat','maxphat')
save('K_D_AC_MinP.dat','minp')
save('K_C_AC_MinP.dat','minphat')
save('K_D_AC_MinLambda.dat','minstretch')
save('K_C_AC_MinLambda.dat','minstretchhat')
save('K_D_AC_MaxLambda.dat','maxstretch')
save('K_C_AC_MaxLambda.dat','maxstretchhat')



%% IN SCRIPT FUNCTIONS

%% Discrete ODE

function dudt = NHBdry_ODE_AC(t,u)

% % Stretches
stretch = (u(2:N)-u(1:N-1))'./L0(1:N-1);
stretch(N) = (u(1) + N*L0(N) - u(N))/L0(N);
% Cell width
Width = 1./stretch; 

% % Active Contraction
% beta 
bet= -1*ones(1,N);
% frequency of contractile function
omega = 2*pi*omega1;
% phase of contractile function
tr = 2*pi*(x0)/N;
% Initialise active function
active=zeros(1,N);
for itd1 = 1:N
    active(itd1) = 1+(sin((omega*t-tr(itd1))));
end

% % Force balance at boundaries
% Initialise force vector
f = zeros(N+1,1); 
% Width used in force balance - shared boundary of cells
Wforce=min([Width(2:N); Width(1:N-1)]);
% Forces
f(2:N) = gam0*Wforce.*(mu(1:N-1).*...
    (stretch(1:N-1).^(-2)-stretch(1:N-1).^(2))...
    +mu(2:N).*(stretch(2:N).^(2)-stretch(2:N).^(-2))...
    +active(1:N-1).*(ones(1,N-1)+bet(1:N-1).*(stretch(1:N-1)-1))...
    -active(2:N).*(ones(1,N-1) + bet(2:N).*(stretch(2:N)-1)));
% Periodic BC
WforceEND=min(Width(N),Width(1));
f(N+1) = gam0*WforceEND*(mu(N)*(stretch(N)^(-2)-stretch(N)^(2))...
    +mu(1)*(stretch(1)^(2)-stretch(1)^(-2))...
    +active(N).*(1 + bet(N).*(stretch(N) - 1))...
    -active(1).*(1 + bet(1).*(stretch(1)-1)));
f(1)=f(N+1);

% Derivative vector
dudt = (1/K)*f;

end


%% Stokes Dissipation with Growth : Continuum PDE

function dUdT = NHBdry_contODE_AC(T,U)


% % Active Contraction
% beta 
betahat = -1*ones(1,nc);
% frequency of contractile function
omegahat = 2*pi*omega1;
% phase of contractile function
trhat = 2*pi*X0/N;
% Initialise active function
activehat=zeros(1,nc);
for itc1 = 1:nc
    activehat(itc1) = 1+(sin((omegahat*T-trhat(itc1))));
end
ghat = activehat;

% % Initialise derivative vector
dUdT = zeros(size(U));

for itc2 = 2:nc-1
    % Discretisation scheme for PDE 
    dUdT(itc2) = gam0*((2*muhat(itc2)*l0)*((1/(dX*dX))...
        *(U(itc2+1)-2*U(itc2)+U(itc2-1)))...
        *(1 + ((1/(2*dX))*(U(itc2+1)-U(itc2-1)))^(-4)) + ...
       (l0)*(1/(2*dX))*(muhat(itc2+1)-muhat(itc2-1))*...
        (1/(2*dX))*(U(itc2+1)-U(itc2-1))*...
        (1 - ((1/(2*dX))*(U(itc2+1)-U(itc2-1)))^(-4))-...
        (l0)*((1/(2*dX))*(U(itc2+1)-U(itc2-1)))^(-1)*...
        ((1/(2*dX))*(ghat(itc2+1)-ghat(itc2-1))*...
        (1+betahat(itc2)*((1/(2*dX))*(U(itc2+1)-U(itc2-1))-1))...
        +ghat(itc2)*betahat(itc2)*((1/(dX*dX))*...
        (U(itc2+1)-2*U(itc2)+U(itc2-1)))));
    
end

% % Boundary conditions
% Periodicity
dUdT(nc) = gam0*((2*muhat(1)*l0)*((1/(dX*dX))*...
        (U(2)-2*U(1)+U(nc-1)-N*l0))...
        *(1 + ((1/(2*dX))*(U(2)-(U(nc-1)-N*l0)))^(-4)) + ...
       (l0)*(1/(2*dX))*(muhat(2)-muhat(nc-1))*...
        (1/(2*dX))*(U(2)-(U(nc-1)-N*l0))*...
        (1 - ((1/(2*dX))*(U(2)-(U(nc-1)-N*l0)))^(-4))-...
        (l0)*((1/(2*dX))*(U(2)-(U(nc-1)-N*l0)))^(-1)*...
        ((1/(2*dX))*(ghat(2)-ghat(nc-1))*...
        (1+betahat(1)*((1/(2*dX))*(U(2)-(U(nc-1)-N*l0))-1))+...
        ghat(1)*betahat(1)*((1/(dX*dX))*...
        (U(2)-2*U(1)+(U(nc-1)-N*l0)))));
dUdT(1) = dUdT(nc);

dUdT = dUdT/K;

end

end
