%% Growth (With K)
% Roxanna Barry
% University of Glasgow
%
% Numerical Solution for single line of Neo-Hookean 
% incompressible cells, with stress-dependent growth and 
% substrate dissipation.
%
% Either Free boundary condition, or constrained boundary
% can be chosen in the ODE and PDE functions at the end of
% the code (make sure to change in both the ode solvers and 
% the initial condition solvers for a0)
%
% kappa = A is defined, for kappa=1 follow instructions in 
% solver comments at the end of the code

function GrowthWithK

tic

%% System Parameters

%
% % Discrete System
%

% Number of cells
N = 100;
% Shear modulus
mu = 1;
% Resting/initial width of cells
W0=1;
% Resting/initial length of array
l0 = N;
% Non-dimensional length-width ratio
gam0 = W0;
% % Number of points in parameter space plots
param = 50;
% Non-dimensional Stokes Damping Coefficient
Ks = linspace(1e-5,1,param);
Ks(2) = 0.01;
Ks(6) = 0.1;
Ks = 1*Ks;
% Non-dimensional Kelvin Dissipation Coefficient
% % linspace
etas = linspace(1e-5,1,param);
etas(2) = 0.01;
etas(6) = 0.1;
etas = 0*etas;

% Time Span
dt = 0.01;
tspan = 0:dt:10;

% Initialise matrix storage for data
maxp = zeros(length(tspan),param);
minp = zeros(length(tspan),param);
minalpha = zeros(length(tspan),param);
maxsig = zeros(length(tspan),param);
minsig = zeros(length(tspan),param);
maxg1 = zeros(length(tspan),param);
ming1 = zeros(length(tspan),param);
L = zeros(length(tspan),param);

%
% % Continuum System
%
% Discretisation Points
nc = 101;
% Discretisation Step Size
dX = l0/(nc-1);
% Continuum Time Span
Tspan = tspan;


% % Initialise matrix storage for data
maxphat = zeros(length(Tspan),param);
minphat = zeros(length(Tspan),param);
minalphahat = zeros(length(tspan),param);
maxsighat = zeros(length(tspan),param);
minsighat = zeros(length(tspan),param);
maxg1hat = zeros(length(tspan),param);
ming1hat = zeros(length(tspan),param);
Lhat = zeros(length(Tspan),param);

%% Initial Conditions

%
% % Discrete System
%

% Cell boundary locations
x0=0:1:l0;
% Initial Condition growth
g0 = ones(1,N);
% Define homeostatic stress value
sig0 = 1;
% Initial guess for a=dalpha/dt
a0 = -0.5*ones(1,N);

%
% % Continuum System
%

% Discretisation point locations
X0 = 0:dX:l0;
% Initial Condition growth
G0 = ones(1,nc);
% Initial guess for ahat=dalphahat/dt
A0 = -0.5*ones(1,nc);


%% Loop over K/eta values

for loop = 1:param
    
    % Non-dimensional Stokes Damping Coefficient
    K = Ks(loop);
    % Non-dimensional Kelvin Dissipation Coefficient
    eta = etas(loop);
    
    % % Initial Condition
    % Discrete
    % Solve for initial guess consistent with system
    a = fsolve(@ICdisc,a0);
    % Full Initial condition of x and g
    ic = [x0,g0,a];
    % Continuum
    % Solve for initial guess consistent with system
    A = fsolve(@ICcont,A0);
    % Full Initial condition of X and G
    IC = [X0,G0,A];

%% Solve the ODE/PDE System

%
% % Discrete System 
%

% Alter mass matrix to account for algebraic constraint
% for d(alpha)/dt
m = eye(3*N+1,3*N+1);
for mit = 2*N+2:3*N+1
    m(mit,mit) = 0;
end
% If eta = 10 use values: a=1e-8, r=1e-4;
% else use: a=1e-4, r=1e-3.
a = 1e-6;
r = 1e-4;
options = odeset('Mass',m,'AbsTol',a,'RelTol',r);
[t,x] = ode23t(@NHBdry_ODE_SG_AW, tspan, ic,options);

%
% % Continuum System 
%

% Alter mass matrix to account for algebraic constraint
% for d(alpha)/dt
M = eye(3*nc,3*nc);
for Mit = 2*nc+1:3*nc
    M(Mit,Mit) = 0;
end
% If eta = 10 use values: a=1e-8, r=1e-4;
% else use: a=1e-4, r=1e-3.
A = 1e-6;
R = 1e-4;
Options = odeset('Mass',M,'AbsTol',A,'RelTol',R);
[T,X] = ode15s(@NHBdry_contODE_SG_AW, Tspan, IC,Options);


%% Growth 

% % Discrete
g1 = x(:,N+2:2*N+1);
maxg1(:,loop) = max(g1,[],2);
ming1(:,loop) = min(g1,[],2);

% % % Continuum
g1hat = X(:,nc+1:2*nc);
maxg1hat(:,loop) = max(g1hat,[],2);
ming1hat(:,loop) = min(g1hat,[],2);

%% Formulating Data to Save

%
% % Discrete System
%

xcpos=zeros(length(t),N);
for it4=1:N
    xcpos(:,it4) = (x(:,it4+1) + x(:,it4))/2;
end

% Array Length
L(:,loop) = x(:,N+1);


% % Continuum System : Average Widths
Xpos = X;
for it6=1:nc
    Xpos(:,it6) = X(:,it6) - X(:,1);
end
X=Xpos;

% Array Length
Lhat(:,loop) = X(:,nc);

%
% % Pressure and Stress and Elastic Stretch
%

sig = zeros(length(Tspan),N);
p = zeros(length(Tspan),N);
alpha = zeros(length(Tspan),N);
sighat = zeros(length(Tspan),nc);
phat = zeros(length(Tspan),nc);
alpht = zeros(length(Tspan),nc);

for i = 1:length(t)

    [s, pe, aa, dg] = NHBdry_ODE_SG_AW_psig(t(i),x(i,:)');

    [sh,ph,ah,dgh] = NHBdry_contODE_SG_AW_psighat(T(i),X(i,:)');
    
    sig(i,:) = s;
    p(i,:) = pe;
    alpha(i,:) = aa;
    dg1dt(i,:) = dg;
    
    sighat(i,:) = sh;
    phat(i,:) = ph;
    alpht(i,:) = ah;
    dg1hatdt(i,:) = dgh;
    
end

% Discrete
minalpha(:,loop) = min(alpha,[],2);
maxp(:,loop) = max(p,[],2);
minp(:,loop) = min(p,[],2);
maxsig(:,loop) = max(sig,[],2);
minsig(:,loop) = min(sig,[],2);

% Continuum
minalphahat(:,loop) = min(alpht,[],2);
maxphat(:,loop) = max(phat,[],2);
minphat(:,loop) = min(phat,[],2);
maxsighat(:,loop) = max(sighat,[],2);
minsighat(:,loop) = min(sighat,[],2);


% Save data for each simulation
save(['K' num2str(K) 'eta' num2str(eta) '_SGD_D_x.dat'],'x')
save(['K' num2str(K) 'eta' num2str(eta) '_SGC_C_x.dat'],'X')
save(['K' num2str(K) 'eta' num2str(eta) '_SGD_D_alph.dat'],'alpha')
save(['K' num2str(K) 'eta' num2str(eta) '_SGC_C_alph.dat'],'alpht')
save(['K' num2str(K) 'eta' num2str(eta) '_SGD_D_sig.dat'],'sig')
save(['K' num2str(K) 'eta' num2str(eta) '_SGC_C_sig.dat'],'sighat')
save(['K' num2str(K) 'eta' num2str(eta) '_SGD_D_p.dat'],'p')
save(['K' num2str(K) 'eta' num2str(eta) '_SGC_C_p.dat'],'phat')
save(['K' num2str(K) 'eta' num2str(eta) '_SGD_D_t.dat'],'t')
save(['K' num2str(K) 'eta' num2str(eta) '_SGC_C_T.dat'],'T')


% keep track of how many simulations have been completed & saved
disp(['Sim ' num2str(loop) ' of ' num2str(param) ' completed'])

end

% % Save Parameter Data

save('D_SG_Ks.dat','Ks')
save('D_SG_etas.dat','etas')
save('D_SG_MaxP.dat','maxp')
save('C_SG_MaxP.dat','maxphat')
save('D_SG_MinP.dat','minp')
save('C_SG_MinP.dat','minphat')
save('D_SG_MinAlpha.dat','minalpha')
save('C_SG_MinAlpha.dat','minalphahat')
save('D_SG_MaxSig.dat','maxsig')
save('C_SG_MaxSig.dat','maxsighat')
save('D_SG_MinSig.dat','minsig')
save('C_SG_MinSig.dat','minsighat')
save('D_SG_Maxg1.dat','maxg1')
save('C_SG_Maxg1.dat','maxg1hat')
save('D_SG_Ming1.dat','ming1')
save('C_SG_Ming1.dat','ming1hat')
save('D_SG_L.dat','L')
save('C_SG_Lhat.dat','Lhat')


toc




%% IN SCRIPT FUNCTIONS

%% Discrete ODE

function dudt = NHBdry_ODE_SG_AW(~,u)

% Define growth
growth1 = u(N+2:2*N+1);
% Define boundary locs
xb = u(1:N+1);

% % Cell stretches
stretch = (xb(2:N+1)-xb(1:N));
alph = (stretch./growth1);
% Cell width (incompressibility constraint)
growth2 = 1;
Width = growth2*gam0./alph; 

% % Define dgdt
fg1 = 1/2*(...
    1 + tanh(sig0*(...
    mu*(alph.^2 - alph.^(-2)) ...
    + eta*(1 + alph.^(-2)).*u(2*N+2:3*N+1) ...
    ))...
    );

% % define dxdt
% Shared boundary width
Wforce=1/2*(Width(1:N-1)+Width(2:N));
% Initialise Force Vector for dxbdt
fxb = zeros(length(xb),1); 
% Force balance at boundaries
fxb(2:N) = -Wforce.*(mu*(alph(1:N-1).^2 - alph(1:N-1).^(-2))...
    - mu*(alph(2:N).^2 - alph(2:N).^(-2))...
    + eta*(...
    + u(2*N+2:3*N).*( 1 + alph(1:N-1).^(-2) ) ...
    - u(2*N+3:3*N+1).*( 1 + alph(2:N).^(-2) ) ...
    )...
    );

% Free boundary (for fixed boundary, comment out)
fxb(N+1) = -Width(N)*(...
    mu*(alph((N))^(2)-alph((N))^(-2))...
    + eta*( 1+alph(N)^(-2) )*u(3*N+1) ...
    );

% New force for COM formulation
fs = zeros(N+1,1);
fs(2:N+1) = fxb(1:N)+fxb(2:N+1);

% Mass matrix
% for kappa=K ::Mxb(q,q)=Mxb(q,q-1) = K
% for kappa=K*A ::Mxb(q,q)=Mxb(q,q-1)=K*growth1(q-1)*growth2
Mxb = eye(N+1,N+1);
for q = 2:N+1
    Mxb(q,q) = K*growth1(q-1)*growth2;
    Mxb(q,q-1) = K*growth1(q-1)*growth2;
end

% define dxdt
dxdt = Mxb\fs;

% % Define relation for d(alpha)/dt, g1, x
falpht = u(2*N+2:3*N+1) ...
    - growth1.^(-1).*((fxb(2:N+1)-fxb(1:N))) ...
    + alph./growth1.*fg1;

% Define full dudt
dudt = zeros(length(u),1);
dudt(1:N+1) = dxdt;
dudt(N+2:2*N+1) = fg1;
dudt(2*N+2:3*N+1) = falpht;

end


%% Continuum PDE

function dUdT = NHBdry_contODE_SG_AW(~,U)

% Define growth
gro1h = U(nc+1:2*nc);
gro2h= 1;
% Define boundary locs
xbhat = U(1:nc);

% Spatial derivative stencils
% x
dxbhatdx = zeros(nc,1);
dxbhatdx(2:nc-1) = (xbhat(3:nc) - xbhat(1:nc-2))/2/dX;
dxbhatdx(1) = ( -3*xbhat(1) + 4*xbhat(2) - xbhat(3) )/2/dX;
dxbhatdx(nc) = (3*xbhat(nc) - 4*xbhat(nc-1) + xbhat(nc-2))/2/dX;

% alpha
alphht = dxbhatdx./gro1h;

dalphhtdx = zeros(nc,1);
dalphhtdx(2:nc-1) = (alphht(3:nc) - alphht(1:nc-2))/2/dX;
dalphhtdx(1) = (-3*alphht(1) + 4*alphht(2) - alphht(3))/2/dX;
dalphhtdx(nc) = (3*alphht(nc)-4*alphht(nc-1)+alphht(nc-2))/2/dX;

dalphhtdtdx = zeros(nc,1);
dalphhtdtdx(2:nc-1) = (U(2*nc+3:3*nc) - U(2*nc+1:3*nc-2))/2/dX;
dalphhtdtdx(1) = (-3*U(2*nc+1) + 4*U(2*nc+2) - U(2*nc+3))/2/dX;
dalphhtdtdx(nc) = (3*U(3*nc) - 4*U(3*nc-1) + U(3*nc-2))/2/dX;

% % Initialise Force Vector for dg1dt
fg1hat = 1/2*(...
    1 + tanh(sig0*(...
    mu*(alphht.^2 - alphht.^(-2)) ...
    + eta*(1 + alphht.^(-2)).*U(2*nc+1:3*nc) ...
    ))...
    );

% % Initialise Force Vector for dxbdt
fxbhat = zeros(length(xbhat),1); 
% Force balance at boundaries
fxbhat(2:nc-1) = gro2h*gam0./alphht(2:nc-1).*...
    (  2*mu*( (alphht(3:nc) - alphht(1:nc-2))/2/dX ) ...
    .*( alphht(2:nc-1) + alphht(2:nc-1).^(-3) ) ...
    + eta*((1 + alphht(2:nc-1).^(-2)).*...
    dalphhtdtdx(2:nc-1) - 2*alphht(2:nc-1).^(-3)...
    .*dalphhtdx(2:nc-1).*U(2*nc+2:3*nc-1)) );
% Free boundary (for fixed boundary, comment out)
fxbhat(nc) = -gam0*gro2h/alphht(nc)*...
    (mu*(alphht(nc)^2 - alphht(nc)^(-2))...
    + eta*( (1+alphht(nc)^(-2))*U(3*nc)  ) );
% for kappa=K ::fxbhat = fxbhat/K;
% for kappa=K*A ::fxbhat = fxbhat/K./gro1h(2:nc-1)/gro2h;
fxbhat = fxbhat/K./gro1h/gro2h;

% Alg constraint for d(alpha)/dt
fahat = zeros(nc,1); 
fahat(2:nc-1) = U(2*nc+2:3*nc-1) - 1/2/dX*(fxbhat(3:nc) ...
    - fxbhat(1:nc-2))./gro1h(2:nc-1)...
    + gro1h(2:nc-1).^(-1).*fg1hat(2:nc-1).*alphht(2:nc-1);
fahat(1) = U(2*nc+1) ...
    - 1/2/dX*( -3*fxbhat(1)+4*fxbhat(2)-fxbhat(3) )/...
    gro1h(1)+ gro1h(1)^(-1)*fg1hat(1)*alphht(1);
fahat(nc) = U(3*nc) ...
    - 1/2/dX*( 3*fxbhat(nc)-4*fxbhat(nc-1)+fxbhat(nc-2) )/...
    gro1h(nc)+ gro1h(nc)^(-1)*fg1hat(nc)*alphht(nc);

% % Next define full dudt
dUdT = zeros(length(U),1);
dUdT(1:nc) = fxbhat;
dUdT(nc+1:2*nc) = fg1hat;
dUdT(2*nc+1:3*nc) = fahat;

end


%% Discrete Pressure, Stress, Stretch and Growth Rate

function [st, pr, al, fg1] = NHBdry_ODE_SG_AW_psig(~,u)

% Define growth
growth1 = u(N+2:2*N+1);
% Define boundary locs
xb = u(1:N+1);

% % Cell stretches
stretch = (xb(2:N+1)-xb(1:N));
al = (stretch./growth1);


st =   mu*(al.^2 - al.^(-2)) ...
    + eta*(1 + al.^(-2)).*u(2*N+2:3*N+1);

pr = mu*al.^(-2) + eta*al.^(-2).*u(2*N+2:3*N+1);

fg1 = 1/2*(...
    1 + tanh(sig0*(...
    mu*(al.^2 - al.^(-2)) ...
    + eta*(1 + al.^(-2)).*u(2*N+2:3*N+1) ...
    ))...
    );


end


%% Continuum Pressure, Stress, Stretch and Growth Rate

function [sth, prh, alh, fg1h] = NHBdry_contODE_SG_AW_psighat(~,U)

% Define growth
growth1hat = U(nc+1:2*nc);
% Define boundary locs
xbhat = U(1:nc);

% First Derivative
% x
dxbhatdx = zeros(nc,1);
dxbhatdx(2:nc-1) = (xbhat(3:nc) - xbhat(1:nc-2))/2/dX;
dxbhatdx(1) = (-3*xbhat(1) + 4*xbhat(2) - xbhat(3))/2/dX;
dxbhatdx(nc) = (3*xbhat(nc) - 4*xbhat(nc-1) + xbhat(nc-2))/2/dX;

% alpha
alh = dxbhatdx./growth1hat;

sth =   mu*(alh.^2 - alh.^(-2)) ...
    + eta*(1 + alh.^(-2)).*U(2*nc+1:3*nc);

prh = mu*(alh.^(-2)) + eta*alh.^(-2).*U(2*nc+1:3*nc);

fg1h = 1/2*(...
    1 + ...
      tanh(sig0*(...
    mu*(alh.^2 - alh.^(-2)) ...
    + eta*(1 + alh.^(-2)).*U(2*nc+1:3*nc) ...
    ))...
    );


end

%% Discrete Initial Condition for Alpha

function dudt = ICdisc(u)

% Define growth
growth1 = g0;
% Define boundary locs
xb =x0;

% % Cell stretches
stretch = (xb(2:N+1)-xb(1:N));
alph = (stretch./growth1);
% Cell width (incompressibility constraint)
growth2 = 1;
Width = growth2*gam0./alph; 

% % Define dgdt
fg1 = 1/2*(...
    1 + tanh(sig0*(...
    mu*(alph.^2 - alph.^(-2)) ...
    + eta*(1 + alph.^(-2)).*u ...
    ))...
    );

% % define dxdt
% Shared boundary width
Wforce=1/2*(Width(1:N-1)+Width(2:N));
% Initialise Force Vector for dxbdt
fxb = zeros(length(xb),1); 
% Force balance at boundaries
fxb(2:N) = -Wforce.*(mu*(alph(1:N-1).^2 - alph(1:N-1).^(-2))...
    - mu*(alph(2:N).^2 - alph(2:N).^(-2))...
    + eta*(...
    + u(1:N-1).*( 1 + alph(1:N-1).^(-2) ) ...
    - u(2:N).*( 1 + alph(2:N).^(-2) ) ...
    )...
    );

% Free boundary (for fixed boundary, comment out)
fxb(N+1) = -Width(N)*(...
    mu*(alph((N))^(2)-alph((N))^(-2))...
    + eta*( 1+alph(N)^(-2) )*u(N) ...
    );

% can use kappa = K for initial condition
fxb=fxb'/K;

% % Define relation for d(alpha)/dt, g1, x
falpht = u ...
    - growth1.^(-1).*((fxb(2:N+1)-fxb(1:N))) ...
    + alph./growth1.*fg1;

% Next define dudt
dudt = falpht;

end


%% Continuum Initial Condition for Alpha

function dUdT = ICcont(U)

% Define growth
gro1h = G0;
gro2h= 1;
% Define boundary locs
xbhat = X0;

% First Derivative
% x
dxbhtdx = zeros(nc,1)';
dxbhtdx(2:nc-1) = (xbhat(3:nc)-xbhat(1:nc-2))/2/dX;
dxbhtdx(1) = (-3*xbhat(1)+4*xbhat(2)-xbhat(3))/2/dX;
dxbhtdx(nc) = (3*xbhat(nc)-4*xbhat(nc-1)+xbhat(nc-2))/2/dX;

% alpha
alphht = dxbhtdx./gro1h;
dalphhtdx = zeros(nc,1)';
dalphhtdx(2:nc-1)=(alphht(3:nc)-alphht(1:nc-2))/2/dX;
dalphhtdx(1) =(-3*alphht(1)+4*alphht(2)-alphht(3))/2/dX;
dalphhtdx(nc)=(3*alphht(nc)-4*alphht(nc-1)+alphht(nc-2))/2/dX;

% dalpha/dt
dalphhtdtdx = zeros(nc,1)';
dalphhtdtdx(2:nc-1) = (U(3:nc) - U(1:nc-2))/2/dX;
dalphhtdtdx(1) = ( -3*U(1) + 4*U(2) - U(3) )/2/dX;
dalphhtdtdx(nc) = ( 3*U(nc) - 4*U(nc-1) + U(nc-2) )/2/dX;

% % Initialise Force Vector for dg1dt
fg1hat = 1/2*(...
    1 + tanh(sig0*(...
    mu*(alphht.^2 - alphht.^(-2)) ...
    + eta*(1 + alphht.^(-2)).*U ...
    ))...
    );

% % Initialise Force Vector for dxbdt
fxbhat = zeros(length(xbhat),1); 
% Force balance at boundaries
fxbhat(2:nc-1) = gro2h*gam0./alphht(2:nc-1).*...
    (  2*mu*( (alphht(3:nc) - alphht(1:nc-2))/2/dX ) ...
    .*( alphht(2:nc-1) + alphht(2:nc-1).^(-3) ) ...
    + eta*(...
    (1 + alphht(2:nc-1).^(-2)).* dalphhtdtdx(2:nc-1) ...
    - 2*alphht(2:nc-1).^(-3).*dalphhtdx(2:nc-1).*U(2:nc-1) ...
    ) );
% Free boundary (for fixed boundary, comment out)
fxbhat(nc) = -gam0*gro2h/alphht(nc)*...
    (mu*(alphht(nc)^2 - alphht(nc)^(-2))...
    + eta*( (1+alphht(nc)^(-2))*U(nc)  ) );

% can use kappa=K for initial condition;
fxbhat = fxbhat'/K;

% Alg constraint for d(alpha)/dt
fahat = zeros(nc,1)'; 
fahat(2:nc-1) = U(2:nc-1) - 1/2/dX*(fxbhat(3:nc) ...
    - fxbhat(1:nc-2))./gro1h(2:nc-1)...
    + gro1h(2:nc-1).^(-2).*fg1hat(2:nc-1).*dxbhtdx(2:nc-1);
fahat(1) = U(1) ...
    - 1/2/dX*( -3*fxbhat(1)+4*fxbhat(2)-fxbhat(3) )/...
    gro1h(1)+ gro1h(1)^(-2)*fg1hat(1)*dxbhtdx(1);
fahat(nc) = U(nc) ...
    - 1/2/dX*( 3*fxbhat(nc)-4*fxbhat(nc-1)+fxbhat(nc-2) )/...
    gro1h(nc)+ gro1h(nc)^(-2)*fg1hat(nc)*dxbhtdx(nc);

% Next define dudt
dUdT = fahat;

end

end