%% Growth With Proliferation (Discrete System Only)
% Roxanna Barry
% University of Glasgow
%
% Numerical Solution for single line of Neo-Hookean 
% incompressible cells, with stress-dependent growth and 
% substrate dissipation and a free boundary.
%
% kappa = A is defined, for kappa=1 follow instructions in 
% solver comments at the end of the code

function GrowthWithProliferation

%% System Parameters

% Number of points in parameter space plots
param = 50;
% Non-dimensional Stokes Damping Coefficient
Ks = linspace(1e-5,1,param);
etas = linspace(1e-5,1,param);

% Shear modulus 
mu = 1;

% Time span
TEND = 101;
dt = 0.01;
tspan0 = 0:dt:TEND;
TSl = length(tspan0);

% Initialise Storage Matrices
Nt = NaN(param,TSl*2);
Lt = NaN(param,TSl);
DivNo = NaN(param,1);
tStore = NaN(param,TSl*2);
DivLocStore = NaN(param,TSl);
DivLocTStore = NaN(param,TSl);

minalpha = NaN(param,1);
maxpressure = NaN(param,1);
minpressure = NaN(param,1);
maxg1 = NaN(param,1);
ming1 = NaN(param,1);

LDIVs = NaN(param,TSl);
LRDIVs = NaN(param,TSl);

%% Loop over paramter values 

for loop = 1:param

% Substrate damping coefficient
K = Ks(loop);
% Viscous damping coefficient
eta = etas(loop);

% Initialise time span
tspan=tspan0;

%% Initial System

% Initial number of cells
N0 = 10;
% Current number of cells
N = N0;
% Resting/initial length of cells
L0 = ones(1,N0);
% Resting/initial width of cells
W0=1;
% Resting/initial length of array
l0 = L0(1)*N;
% Non-dimensional length-width ratio
gam0 = W0/L0(1);
% Counter for number of divisions
divcount = 0;
% Position of cell that has divided
xdivloc = [];


t = 0;
% Initialise division times storage matrix
tends = [];
tlends = 0;
tslends = 0;
% Initialise time storage matrix
tstore = [];
% Initialise tdiv (previous dividing time, for growth 
% function to reset to t=0)
tdiv=zeros(N0,1);

%% Initial Condition

% cell boundary locations
x0=0:L0:l0;
% initialise storage matrix for cell boundary locations
xstore = [];
% Initial Condition growth
g0 = ones(1,N);
growth1store = [];
% Define homeostatic stress value
sig0 = 10;
% Define max growth rate
GA = 1;
a0 = -0.5*ones(1,N);
% Full Initial condition of x and g and dalphadt
ic = [x0, g0, a0];

% Resting lengths of current cells
L0 = ones(1,N0);
% Store resting lengths of cells at each division
L0store = L0;
% Initialise storage matrices for 
% length
L = [];
% number of cells
Ntl = [];
% length of array at division
Ldivs = [];
% ratio of division location to array length
LRdivs = [];
% pressure
pressure = [];
% stretch
alpha = [];

%% Solving Loop

while t(end) < TEND-1 && N <= 1000
    
    % Cell widths
    W0 = ones(1,N);
    
    % Solving ODE
    m = eye(3*N+1,3*N+1);
    % Alter mass matrix to account for algebraic constraint
    % for d(alpha)/dt
    for mit = 2*N+2:3*N+1
        m(mit,mit) = 0;
    end
    a = 1e-6;
    r = 1e-6;
    options = odeset('Mass',m,'AbsTol',a,'RelTol',r,...
        'Events',@EventsFn);
    [t,x,~,~,ie] = ode15s(@NHBdry_Subs_ODE_GrowthDiv,...
        tspan, ic, options);
    
    % % Store information
    
    % If division event has occurred, store information
    if ie
        tends = [ tends t(end) ];
        xdivloc = [xdivloc x(end,ie)];
        Ldivs = [Ldivs x(end,N+1)];
        LRdivs = [LRdivs x(end,N)];
    end
    
    % store times at which code has stopped
    tlends = [ tlends length(t)+tlends(end) ];
    if divcount == 0
    	tslends = [tslends length(t)+tslends(end)];
    else
        tslends = [tslends length(t)+tslends(end)-1];
    end
    
    % store time values
    tstore = [ tstore ; t ];
    % store boundary locations
    [xstore0,~] = size(xstore);
    xstore = [ [ xstore  NaN(xstore0,1) ] ; x(:,1:N+1) ];
    lie=length(ie);
    if lie>1
        if all(ie == ie(1))
            ie = ie(1);
        else
        disp('Multiple divisions deteced at same time :(')
        return
        end
    end
    
    % store growth values
    [growth10,~] = size(growth1store);
    A = [growth1store NaN(growth10,1) ];
    % stress dependant growth
    growth1store = [A ; x(:,N+2:2*N+1)];
    
    % store alpha and pressure
    pee = zeros(length(t),N);
    aah = zeros(length(t),N);
    for i = 1:length(t)
        [~,pee(i,:),aah(i,:),~]=NHBdry_ODE_SGP_psig(t(i),x(i,:));
    end
    [p1, ~] = size(pressure);
    pressure = [ pressure NaN(p1,1) ; pee];
    [a1,~] = size(alpha);
    alpha = [alpha NaN(a1,1); aah];
    
    % Set up initial conditions for next loop iteration
    % to reflect new cell
    tdiv = [tdiv(1:ie-1); t(end); t(end); tdiv(ie+1:end)];
    tspan = t(end):0.01:TEND;
    x0=[x(end,1:ie) (x(end,ie)+x(end,ie+1))/2 x(end,ie+1:N+1)];
    g0 = [x(end,N+2:N+2+ie-1) 1 x(end,N+2+ie:2*N+1)];
    g0(ie)=1;
    a0 = [x(end,2*N+2:2*N+2+ie-1) -0.5 x(end,2*N+2+ie:end)];
    a0(ie)=-0.5;
    ic = [x0,g0,a0]; 
    
    % Store array length
    L = [L ; x(:,N+1)];
    % Store number of cells at current time
    Ntl = [Ntl; N*ones(length(t),1)];
    % Current number of cells
    N = N + 1;
    
    % Update resting cell lenghts 
    if ie
        [a,~] = size(L0store);
        L0old = [ L0store zeros(a,1) ];
        L0 = [ L0(1:ie-1) ...
            1/2*L0(ie)*growth1store(end,ie) ...
            1/2*L0(ie)*growth1store(end,ie) ...
            L0(ie+1:end)] ;
        L0store = [ L0old ; L0 ];
        divcount = divcount + 1;
    end
end

%% Defining properties of System over parameters

% % Number of cells and length of array over time
Nt(loop,1:length(Ntl)) = Ntl;
Lt(loop,1:length(L)) = L;
% Length of array at division
LDIVs(loop,1:length(Ldivs)) = Ldivs;
% Ratio of division location to array length
LRDIVs(loop,1:length(LRdivs)) = LRdivs;

% Maximum and Minimum Values (for tspan only)
minalpha(loop,1:length(tstore)) = min(alpha,[],2);
maxpressure(loop,1:length(tstore)) = max(pressure,[],2);
minpressure(loop,1:length(tstore)) = min(pressure,[],2);
maxg1(loop,1:length(tstore)) = max(growth1store,[],2);
ming1(loop,1:length(tstore)) = min(growth1store,[],2);

% time
tStore(loop,1:length(tstore)) = tstore;

% division locations 
DivLocStore(loop,1:length(xdivloc)) = xdivloc;
% division times
DivLocTStore(loop,1:length(tends)) = tends;


%% Saving Data

% Save full simulation data for certain K
if K == 1e-5 || K == 0.01 || K == 0.1 || K == 1
    save(['K' num2str(K) '_D_SGP_x.dat'],'xstore')
    save(['K' num2str(K) '_D_SGP_L.dat'],'L')
    save(['K' num2str(K) '_D_SGP_t.dat'],'tstore')
    save(['K' num2str(K) '_D_SGP_p.dat'],'pressure')
    save(['K' num2str(K) '_D_SGP_alpha.dat'],'alpha')
    save(['K' num2str(K) '_D_SGP_t.dat'],'tstore')
    save(['K' num2str(K) '_D_SGP_g1.dat'],'growth1store')
    save(['K' num2str(K) '_D_SGP_xdiv.dat'],'xdivloc')
    save(['K' num2str(K) '_D_SGP_tdiv.dat'],'tends')
    save(['K' num2str(K) '_D_SGP_tspandivindex.dat'],'tslends')
    save(['K' num2str(K) '_D_SGP_LDiv.dat'],'Ldivs')
    save(['K' num2str(K) '_D_SGP_LRDiv.dat'],'LRdivs')
end

% keep track of how many simulations have been completed & saved
disp(['Sim ' num2str(loop) ' of ' num2str(param) ' completed'])

end

% Save Parameter Data
    save('K_D_SGP_Nt.dat','Nt')
    save('K_D_SGP_Lt.dat','Lt')
    save('K_D_SGP_DivLoc.dat','DivLocStore')
    save('K_D_SGP_DivLocT.dat','DivLocTStore')
    save('K_D_SGP_t.dat','tStore')
    save('K_D_SGP_DivNo.dat','DivNo')
    save('K_D_SGP_tspan','tspan0')
    save('K_D_SGP_MinAlpha.dat','minalpha')
    save('K_D_SGP_MaxPressure.dat','maxpressure')
    save('K_D_SGP_MinPressure.dat','minpressure')
    save('K_D_SGP_Maxg1.dat','maxg1')
    save('K_D_SGP_Ming1.dat','ming1')
    save('K_D_SGP_Ks.dat','Ks')
    save('K_D_SGP_LDivs.dat','LDIVs')
    save('K_D_SGP_LRDivs.dat','LRDIVs')

%% IN SCRIPT FUNCTIONS

%% Discrete ODE

    function dudt = NHBdry_Subs_ODE_GrowthDiv(~,u)
        
        % Current number of cells
        Nfun = (length(u)-1)/3;
        % Growth
        gro1 = u(Nfun+2:2*Nfun+1);
        % Define boundary locations
        xb = u(1:Nfun+1);
        
        % Cell stretches
        stretch = (xb(2:Nfun+1)-xb(1:Nfun));
        alph = (stretch./gro1);
        % Cell width
        gro2 = 1;
        Width = gro2*gam0./alph;
        
        
        % % Define dgdt
        fg1 = 1/2*GA*(...
            1 + tanh(sig0*(...
            mu*(alph.^2 - alph.^(-2)) ...
            + eta*(1 + alph.^(-2)).*u(2*Nfun+2:3*Nfun+1) ...
            ))...
            );
        
        % % define dxdt
        % Shared boundary width
        Wforce=1/2*(Width(1:Nfun-1)+Width(2:Nfun));
        % Initialise Force Vector for dxbdt
        fxb = zeros(length(xb),1);
        % Force balance at boundaries
        fxb(2:Nfun) = -Wforce.*...
            (mu*(alph(1:Nfun-1).^2 - alph(1:Nfun-1).^(-2))...
          - mu*(alph(2:Nfun).^2 - alph(2:Nfun).^(-2))...
          + eta*(...
          + u(2*Nfun+2:3*Nfun).*( 1 + alph(1:Nfun-1).^(-2) ) ...
          - u(2*Nfun+3:3*Nfun+1).*( 1 + alph(2:Nfun).^(-2) ) ...
          )...
          );
        % Free boundary
       fxb(Nfun+1) = -Width(Nfun)*(...
          mu*(alph((Nfun))^(2)-alph((Nfun))^(-2))...
          + eta*( 1+alph(Nfun)^(-2) )*u(3*Nfun+1) ...
          );
      
      % New force for COM formulation
        fs = zeros(N+1,1);
        fs(2:N+1) = fxb(1:N)+fxb(2:N+1);
        
        % Mass matrix
        % for kappa=K :: Mxb(q,q)=Mxb(q,q-1)=K;
        % for kappa=A*K :: Mxb(q,q)=Mxb(q,q-1)=K*gro1(q-1)*gro2;
        Mxb = eye(N+1,N+1);
        for q = 2:N+1
         Mxb(q,q) = K*gro1(q-1)*gro2;
         Mxb(q,q-1) = K*gro1(q-1)*gro2;
        end
        
        % Solve for dxbdt
        dxbdt = Mxb\fs;

        % % Define d(alpha)/dt
        falpht = u(2*Nfun+2:3*Nfun+1) ...
            - gro1.^(-1).*((dxbdt(2:Nfun+1)-dxbdt(1:Nfun))) ...
            + gro1.^(-2).*stretch.*fg1;
        
        % Define full dudt
        dudt = zeros(length(u),1);
        dudt(1:Nfun+1) = dxbdt;
        dudt(Nfun+2:2*Nfun+1) = fg1;
        dudt(2*Nfun+2:3*Nfun+1) = falpht;

end


%% Event Detector

    function [position,isterminal,direction] = EventsFn(~,x)
        
        growt1 = x(N+2:2*N+1);
        
        % division law based on growth function
        % (current area of cell)
        position = growt1 - 2*ones(N,1);
        
        % Halt integration
        isterminal = ones(N,1);
        % The zero can be approached from either direction
        direction = zeros(N,1);
        
    end

%% Discrete Pressure, Stress, Stretch

function [stress,pressure,alph,fg1dt]=NHBdry_ODE_SGP_psig(~,u)

% Define growth
growth1 = u(N+2:2*N+1);
% Define boundary locs
xb = u(1:N+1);

% % Cell stretches
stretch = (xb(2:N+1)-xb(1:N));
alph = (stretch./growth1);


stress =   mu*(alph.^2 - alph.^(-2)) ...
    + eta*(1 + alph.^(-2)).*u(2*N+2:3*N+1);

pressure = mu*alph.^(-2) + eta*alph.^(-2).*u(2*N+2:3*N+1);

fg1dt = 1/2*GA*(...
    1 + tanh(sig0*(...
    mu*(alph.^2 - alph.^(-2)) ...
    + eta*(1 + alph.^(-2)).*u(2*N+2:3*N+1) ...
    ))...
    );

end

end