%% Growth (With K)
% Roxanna Barry
% University of Glasgow
%
% Numerical Solution for single line of Neo-Hookean 
% incompressible cells, with stress-dependent growth without 
% substrate dissipation (K=0).
%
% Either Free boundary condition, or constrained boundary
% can be chosen in the ODE and PDE functions at the end of
% the code (make sure to change in both the ode solvers and 
% the initial condition solvers for a0)

function GrowthWithoutK
close all

tic

%% System Parameters

%
% % Discrete System
%

% Number of cells
N = 100;
% Shear modulus
mu = 1;
% Resting/initial width of cells
W0=1;
% Resting/initial length of array
l0 = N;
% Non-dimensional length-width ratio
gam0 = W0;
% % Number of points in parameter space plots
param = 20;
% Non-dimensional Kelvin Dissipation Coefficient
etas = logspace(-5,2,param);

% Discrete Time Span
dt = 0.01;
tspan = 0:dt:100;

% initialise matrix storage
maxp = zeros(length(tspan),param);
minp = zeros(length(tspan),param);
minalpha = zeros(length(tspan),param);
maxsig = zeros(length(tspan),param);
minsig = zeros(length(tspan),param);
maxg1 = zeros(length(tspan),param);
ming1 = zeros(length(tspan),param);
L = zeros(length(tspan),param);

%
% % Continuum System
%

% Discretisation Points
nc = 101;
% Discretisation Step Size
dX = l0/(nc-1);
% Continuum Time Span
Tspan = tspan;

% % initialise matrix storage
maxphat = zeros(length(Tspan),param);
minphat = zeros(length(Tspan),param);
minalphahat = zeros(length(tspan),param);
maxsighat = zeros(length(tspan),param);
minsighat = zeros(length(tspan),param);
maxg1hat = zeros(length(tspan),param);
ming1hat = zeros(length(tspan),param);
Lhat = zeros(length(Tspan),param);

%% Initial Conditions

%
% % Discrete System
%
a0=ones(1,N);
% Initial Condition growth
g0 = ones(1,N);
% Define homeostatic stress value
sig0 = 1;
% Define max growth rate
GA = 1;
% Full Initial condition of x and g
ic = [a0,g0];

% % Continuum System
A0 = ones(1,nc);
% Initial Condition growth
G0 = ones(1,nc);
% Full Initial condition of X and G
IC = [A0,G0];

%% Loop over K/eta values

for loop = 1:param
    
    % Non-dimensional Kelvin Dissipation Coefficient
    eta = etas(loop);

%% Solve the ODE/PDE System

%
% % Discrete System
%

[t,x] = ode23t(@NHBdry_ODE_SG_AW, tspan, ic);

%
% % Continuum System 
%

[T,X] = ode15s(@NHBdry_contODE_SG_AW, Tspan, IC);


%% Growth 

% % Discrete
g1 = x(:,N+1:2*N);
maxg1(:,loop) = max(g1,[],2);
ming1(:,loop) = min(g1,[],2);

% % % Continuum
g1hat = X(:,nc+1:2*nc);
maxg1hat(:,loop) = max(g1hat,[],2);
ming1hat(:,loop) = min(g1hat,[],2);

%% X values, Stretch and Pressure

%
% % Discrete System 
%

% Elastic stretch
alpha = x(:,1:N);

% Cell boundary locations
xb = zeros(length(t),N+1);
for i = 1:N
    xb(:,i+1) = xb(:,i) + g1(:,i).*alpha(:,i);
end

% Cell centre of mass locations
xcpos=zeros(length(t),N);
for it4=1:N
    xcpos(:,it4) = (xb(:,it4+1) + xb(:,it4))/2;
end
% Array length
L(:,loop) = xb(:,N+1);


%
% % Continuum System 
%

% Elastic stretch
alphahat = X(:,1:nc);

% Node locations
X1 = zeros(length(T),nc);
X1(:,3) = 2*dX*g1hat(:,2).*alphahat(:,2);
X1(:,2) = (2*dX*g1hat(:,1).*alphahat(:,1) + X1(:,3))/4;
for it6=3:nc-1
    X1(:,it6+1)=2*dX*g1hat(:,it6).*alphahat(:,it6)+X1(:,it6-1);
end

% Array length
Lhat(:,loop) = X1(:,nc);


% % Pressure and Stress and Elastic Stretch
sig = zeros(length(Tspan),N);
p = zeros(length(Tspan),N);
dg1dt = zeros(length(Tspan),N);
sighat = zeros(length(Tspan),nc);
phat = zeros(length(Tspan),nc);
dg1hatdt = zeros(length(Tspan),nc);

for i = 1:length(t)

    [s, pe, dg] = NHBdry_ODE_SG_AW_psig(t(i),x(i,:)');

    [sh,peh,dgh]=NHBdry_contODE_SG_AW_psighat(T(i),X(i,:)');
    
    sig(i,:) = s;
    p(i,:) = pe;
    dg1dt(i,:) = dg;
    
    sighat(i,:) = sh;
    phat(i,:) = peh;
    dg1hatdt(i,:) = dgh;
    
end

% Max and Mins 
% Discrete
minalpha(:,loop) = min(alpha,[],2);
maxp(:,loop) = max(p,[],2);
minp(:,loop) = min(p,[],2);
maxsig(:,loop) = max(sig,[],2);
minsig(:,loop) = min(sig,[],2);
% Continuum
minalphahat(:,loop) = min(alphahat,[],2);
maxphat(:,loop) = max(phat,[],2);
minphat(:,loop) = min(phat,[],2);
maxsighat(:,loop) = max(sighat,[],2);
minsighat(:,loop) = min(sighat,[],2);


% Save full simulation data for various eta
if eta == 1e-5 || eta == 1e-3 || eta == 0.1 || eta == 1
    save(['eta' num2str(eta) '_K0_D_SG_x.dat'],'xb')
    save(['eta' num2str(eta) '_K0_D_SG_xc.dat'],'xcpos')
    save(['eta' num2str(eta) '_K0_C_SG_X.dat'],'X1')
    save(['eta' num2str(eta) '_K0_D_SG_p.dat'],'p')
    save(['eta' num2str(eta) '_K0_C_SG_p.dat'],'phat')
    save(['eta' num2str(eta) '_K0_D_SG_alpha.dat'],'alpha')
    save(['eta' num2str(eta) '_K0_C_SG_alpha.dat'],'alphahat')
    save(['eta' num2str(eta) '_K0_D_SG_sig.dat'],'sig')
    save(['eta' num2str(eta) '_K0_C_SG_sig.dat'],'sighat')
    save(['eta' num2str(eta) '_K0_D_SG_g1.dat'],'g1')
    save(['eta' num2str(eta) '_K0_C_SG_g1.dat'],'g1hat')
    save(['eta' num2str(eta) '_K0_D_SG_t.dat'],'t')
    save(['eta' num2str(eta) '_K0_C_SG_T.dat'],'T')
end

% keep track of how many simulations have been completed & saved
disp(['Sim ' num2str(loop) ' of ' num2str(param) ' completed'])

end

% Save Parameter Data 
save('eta_D_SG_etas.dat','etas')
save('eta_D_SG_MaxP.dat','maxp')
save('eta_C_SG_MaxP.dat','maxphat')
save('eta_D_SG_MinP.dat','minp')
save('eta_C_SG_MinP.dat','minphat')
save('eta_D_SG_MinAlpha.dat','minalpha')
save('eta_C_SG_MinAlpha.dat','minalphahat')
save('eta_D_SG_MaxSig.dat','maxsig')
save('eta_C_SG_MaxSig.dat','maxsighat')
save('eta_D_SG_MinSig.dat','minsig')
save('eta_C_SG_MinSig.dat','minsighat')
save('eta_D_SG_Maxg1.dat','maxg1')
save('eta_C_SG_Maxg1.dat','maxg1hat')
save('eta_D_SG_Ming1.dat','ming1')
save('eta_C_SG_Ming1.dat','ming1hat')
save('eta_D_SG_L.dat','L')
save('eta_C_SG_L.dat','Lhat')

toc

%% IN SCRIPT FUNCTIONS

%% Discrete ODE 

function dudt = NHBdry_ODE_SG_AW(~,u)

% stretch
alph = u(1:N);
% growth in the y direction
growth2 = 1;
% cell widths
Width = growth2*gam0./alph; 

% Shared boundary width
Wforce=1/2*(Width(1:N-1)+Width(2:N));
% Initialise Force Vector for dxbdt
fxb = zeros(length(alph),1); 
% Force balance at boundaries
fxb(1:N-1) = -Wforce.*(mu*(alph(1:N-1).^2 - alph(1:N-1).^(-2))...
    - mu*(alph(2:N).^2 - alph(2:N).^(-2))...
    );
% Free boundary (comment out for fixed boundary)
fxb(N) = -Width(N)*(...
    mu*(alph((N))^(2)-alph((N))^(-2))...
    );

% Mass Matrix
Mxb = eye(length(alph),length(alph));
for h = 1:(length(alph)-1)
    Mxb(h,h) = eta*(1 + alph(h)^(-2));
    Mxb(h,h+1) = - eta*(1 + alph(h+1)^(-2));
end
% Free BC
Mxb(N,N) = eta*(1 + alph(N)^(-2));
% Solve for dalpha dt
dadt = Mxb\fxb;

% Define dgdt
fg1 = 1/2*GA*(...
    1 + tanh(sig0*(...
    mu*(alph.^2 - alph.^(-2)) ...
    + eta*(1 + alph.^(-2)).*dadt ...
    ))...
    );

% Define full dudt
dudt = zeros(length(u),1);
dudt(1:N) = dadt;
dudt(N+1:2*N) = fg1;

end


%% Continuum PDE

function dUdT = NHBdry_contODE_SG_AW(~,U)

% Stretch
alphhat = U(1:nc);

% Derivatives
dalphhatdx = zeros(nc,1);
dalphhatdx(2:nc-1) = (alphhat(3:nc) - alphhat(1:nc-2))/2/dX;
dalphhatdx(1) = ( -3*alphhat(1) + 4*alphhat(2) - alphhat(3) )/2/dX;
dalphhatdx(nc) = ( 3*alphhat(nc) - 4*alphhat(nc-1) + alphhat(nc-2) )/2/dX;

% % Initialise Force Vector for dxbdt
fxbhat = zeros(length(alphhat),1); 
% Force balance at boundaries
fxbhat(2:nc-1) = -2*mu/eta*...
    (dalphhatdx(2:nc-1)).*...
    (alphhat(2:nc-1) + alphhat(2:nc-1).^(-3));
% Free boundary (comment out for fixed boundary)
fxbhat(nc) = -mu/eta*(alphhat(nc).^2 - alphhat(nc).^(-2));

% Mass Matrix
Mxbhat = eye(length(alphhat),length(alphhat));
for h = 2:(length(alphhat)-1)
    Mxbhat(h,h-1) = -(1 + alphhat(h)^(-2))/2/dX;
    Mxbhat(h,h) = 2*alphhat(h).^(-3);
    Mxbhat(h,h+1) = (1 + alphhat(h)^(-2))/2/dX;
end
% Free BC
Mxbhat(nc,nc) = (1 + alphhat(nc)^(-2));
% Solve for dalpha dt
dalphdt = Mxbhat\fxbhat;

% Define dg1dt
fg1hat = 1/2*GA*(...
    1 + tanh(sig0*(...
    mu*(alphhat.^2 - alphhat.^(-2)) ...
    + eta*(1 + alphhat.^(-2)).*dalphdt ...
    ))...
    );

% Define full dudt
dUdT = zeros(length(U),1);
dUdT(1:nc) = dalphdt;
dUdT(nc+1:2*nc) = fg1hat;

end


%% Discrete Pressure and Stress

function [stress, pressure, fg1dt] = NHBdry_ODE_SG_AW_psig(~,u)

alph = u(1:N);

% Initialise Force Vector for dxbdt
fxb = zeros(length(alph),1); 
% Force balance at boundaries
fxb(2:N-1) = mu/eta.*((alph(2:N-1).^2 - alph(2:N-1).^(-2))...
    - (alph(1:N-2).^2 - alph(1:N-2).^(-2)));

% Free boundary
fxb(N) = -mu/eta*( alph((N))^(2)-alph((N))^(-2) );

% Mass Matrix
Mxb = eye(length(alph),length(alph));
for h = 2:(length(alph)-1)
    Mxb(h,h) = 1 + alph(h)^(-2);
    Mxb(h,h+1) = - (1 + alph(h+1)^(-2));
end
% Free BC
Mxb(N,N) = (1 + alph(N)^(-2));
% Solve for dalpha dt
dadt = Mxb\fxb;

stress =   mu*(alph.^2 - alph.^(-2)) ...
    + eta*(1 + alph.^(-2)).*dadt;

pressure = mu*alph.^(-2) + eta*alph.^(-2).*dadt;

fg1dt = 1/2*GA*(...
    1 + tanh(sig0*(...
    mu*(alph.^2 - alph.^(-2)) ...
    + eta*(1 + alph.^(-2)).*dadt ...
    ))...
    );

end


%% Continuum Pressure and Stress

function [stresshat, pressurehat, fg1hatdt] = NHBdry_contODE_SG_AW_psighat(~,U)

alphhat = U(1:nc);

% Derivatives
dalphhatdx = zeros(nc,1);
dalphhatdx(2:nc-1) = (alphhat(3:nc) - alphhat(1:nc-2))/2/dX;
dalphhatdx(1) = ( -3*alphhat(1) + 4*alphhat(2) - alphhat(3) )/2/dX;
dalphhatdx(nc) = ( 3*alphhat(nc) - 4*alphhat(nc-1) + alphhat(nc-2) )/2/dX;

% Initialise Force Vector for dxbdt
fxbhat = zeros(length(alphhat),1); 
% Force balance at boundaries
fxbhat(2:nc-1) = -2*mu/eta*...
    (dalphhatdx(2:nc-1)).*...
    (alphhat(2:nc-1) + alphhat(2:nc-1).^(-3));
% Free boundary
fxbhat(nc) = -mu/eta*(alphhat(nc).^2 - alphhat(nc).^(-2));

% Mass Matrix
Mxbhat = eye(length(alphhat),length(alphhat));
for h = 2:(length(alphhat)-1)
    Mxbhat(h,h-1) = -(1 + alphhat(h)^(-2))/2/dX;
    Mxbhat(h,h) = 2*alphhat(h).^(-3);
    Mxbhat(h,h+1) = (1 + alphhat(h)^(-2))/2/dX;
end
% Free BC
Mxbhat(nc,nc) = (1 + alphhat(nc)^(-2));
% Solve for dalpha dt
dalphdt = Mxbhat\fxbhat;

stresshat =   mu*(alphhat.^2 - alphhat.^(-2)) ...
    + eta*(1 + alphhat.^(-2)).*dalphdt;

pressurehat = mu*(alphhat.^(-2)) + eta*alphhat.^(-2).*dalphdt;

fg1hatdt = 1/2*GA*(...
    1 + ...
      tanh(sig0*(...
    mu*(alphhat.^2 - alphhat.^(-2)) ...
    + eta*(1 + alphhat.^(-2)).*dalphdt ...
    ))...
    );

end

end