%% Prescribed Displacement
% Roxanna Barry
% University of Glasgow
%
% Numerical Solution of Discrete ODEs and Continuum PDEs for 
% single line of Neo-Hookean incompressible cells, for a 
% prescribed displacement BC.

function PrescribedDisplacement


%% System Parameters

%
% % Discrete System
%

% Number of cells
N = 100;
% Prescribed Displacement Amount: 
% (k1*100) percentage decrease of array length
k1=0.1;
% Shear modulus
mu = ones(1,N);
% Resting/initial length of cells
L0 = 1;
% Resting/initial width of cells
W0=1;
% Resting/initial length of array
l0 = L0*N;
% Non-dimensional length-width ratio
gam0 = W0/L0(1);
% % Number of points in parameter space plots
param = 50;
% Non-dimensional Stokes Damping Coefficient
Ks = linspace(1e-5,10,param);
Ks(2) = 0.001;
Ks(3) = 0.1;
Ks(6) = 1; 
% If evaluating eta with K=0, set Ks=0*Ks
Ks = 0*Ks;
% Non-dimensional Kelvin Dissipation Coefficient
etas = linspace(1e-5,10,param);
etas(2) = 0.001;
etas(3) = 0.1;
etas(6) = 1;
% If evaluating K with eta=0, set etas=0*etas 
etas = 1*etas;

% Time Span
dt = 0.01;
tspan = 0:dt:10;

% Initialise Storage Matrices for saving
% Columns are one value of K over all time
maxp = zeros(length(tspan),param);
minp = zeros(length(tspan),param);
minstretch = zeros(length(tspan),param);
L = zeros(length(tspan),param);

%
% % Continuum System
%

% Discretisation Points
nc = 101;
% Discretisation Step Size
dX = l0/(nc-1);
% Shear modulus
muhat = ones(1,nc);
% Continuum Time Span
Tspan = tspan;

% Initialise Storage Matrices
maxphat = zeros(length(tspan),param);
minphat = zeros(length(tspan),param);
minstretchhat = zeros(length(tspan),param);
Lhat = zeros(length(tspan),param);

%
% % Analytical System
%

% number of points
na=101;
dxa = l0/(na-1);
% shear modulus
muA = ones(1,na);
% time span (same as continuum)
TA = Tspan;

% Initialise Storage Matrices
maxpA = zeros(length(TA),param);
minpA = zeros(length(TA),param);
minstretchA = zeros(length(TA),param);
LA = zeros(length(TA),param);

%% Initial Conditions

% % Discrete System
x0=0:L0:l0;

% % Continuum System
X0 = 0:dX:l0;

%% Loop over K,eta values

for loop = 1:param
    
    % Non-dimensional Stokes Damping Coefficient
    K = Ks(loop);
    % Non-dimensional Kelvin Dissipation Coefficient
    eta = etas(loop);

%% Solve 

% % Discrete System ODEs
[t,x] = ode15s(@NHBdry_ODE_PD_AW, tspan, x0);
tend = length(t);

% % Continuum System PDEs
[T,X] = ode15s(@NHBdry_contODE_PD_AW, Tspan, X0);
Tend = length(T);

% % Analytical Solutions
XA0 = linspace(0,l0,na);
DispA = zeros(length(TA),101);
% For K non-zero, eta=0
if eta == 0
    
    ts = K/4/gam0/muhat(1);
    
    for it9=1:length(TA)
        DispA(it9,:) = l0*exp(-TA(it9))...
            *sin(sqrt(ts)*XA0)...
            /sin(sqrt(ts)*l0);
    end
    
    % Displacement
    DispAPlot = -k1*XA0 + k1*DispA;
    % X profile
    XA = XA0 + DispAPlot;
    
end
% for eta non-zero, K=0
if K == 0
    
    for it9=1:length(TA)
        
        if TA(it9) <= 1
                DispA(it9,:) = XA0*(2*TA(it9)^3-3*TA(it9)^2);
        else
            DispA(it9,:) = -XA0;
        end
        
    end
    
    % Displacement
    DispAPlot = k1*DispA;
    % X profile
    XA = XA0 + DispAPlot;
    
end


%% Stretch, Pressure, Array Length

%
% % Discrete System 
%

% Centre of mass locations
xcpos=zeros(tend,N);
for it4=1:N
    xcpos(:,it4) = (x(:,it4+1) + x(:,it4))/2;
end
% Discrete array length
L(:,loop) = x(:,N+1);

% Stretch and minimum stretch (max compression)
lambda = zeros(tend,N); 
for it5 = 1:tend
    lambda(it5,:) = (x(it5,2:N+1)-x(it5,1:N))./L0;
    minstretch(it5,loop) = min(lambda(it5,:));
end

% Pressure and max, min pressure
p = zeros(tend,N);
for jj = 1:tend
    p(jj,:) = NHBdry_ODE_PD_AW_p(t(jj),x(jj,:));
end
maxp(:,loop) = max(p,[],2);
minp(:,loop) = min(p,[],2);


%
% % Continuum System 
%

% Continuum domain length
Lhat(:,loop) = X(:,nc);

% Stretch and minimum stretch (max compression)
lambdahat = zeros(Tend,nc); 
for it7=2:nc-1
    lambdahat(:,it7) = (X(:,it7+1) - X(:,it7-1))/2/dX;
end
lambdahat(:,1) = (-3*X(:,1) + 4*X(:,2) - X(:,3))/2/dX; 
lambdahat(:,nc) = (3*X(:,nc) - 4*X(:,nc-1) + X(:,nc-2))/2/dX;
minstretchhat(:,loop) = min(lambdahat,[],2);

% Pressure and max, min pressure
phat = zeros(Tend,nc);
for kk = 1:tend
    phat(kk,:) = NHBdry_contODE_PD_AW_phat(T(kk),X(kk,:));
end
maxphat(:,loop) = max(phat,[],2);
minphat(:,loop) = min(phat,[],2);


%
% % Analytical System
%

% Length of analytical array
LA(:,loop) = XA(:,end);

% Stretch and minimum stretch (max compression)
lambdaA = zeros(Tend,na); 
for it7=2:na-1
    lambdaA(:,it7) = (XA(:,it7+1) - XA(:,it7-1))/2/dxa; 
end
lambdaA(:,1) = (-3*XA(:,1) + 4*XA(:,2) - XA(:,3))/2/dxa; 
lambdaA(:,na) = (3*XA(:,na) - 4*XA(:,na-1) + XA(:,na-2))/2/dxa;
minstretchA(:,loop) = min(lambdaA,[],2);

% Pressure and max, min pressure
pA = zeros(Tend,na); 
% for K values, eta=0
if eta == 0
    
    ts = K/4/gam0/muhat(1); 
    
    for it9=1:length(TA)
        pA(it9,:) = (1 - 2*k1*((l0*exp(-TA(it9))...
            *sqrt(ts)*cos(sqrt(ts)*XA0)...
            /sin(sqrt(ts)*l0)) - 1))...
            .*(muA + 2*eta*k1*(-(l0*exp(-TA(it9))...
            *sqrt(ts)*cos(sqrt(ts)*XA0)...
            /sin(sqrt(ts)*l0))));
    end
    
    maxpA(:,loop) = max(pA,[],2);
    
end
% for eta values, K=0
if K == 0
    
    for it9=1:length(TA)
        
        if TA(it9) <= 1
            pA(it9,:) = (1+2*k1*(3*TA(it9)^2-2*TA(it9)^3))...
                *(muA+12*eta*k1*(TA(it9)^2-TA(it9)));
        else
            pA(it9,:) = muA*(1+2*k1);
        end
        
    end
    
    maxpA(:,loop) = max(pA,[],2);
    minpA(:,loop) = min(pA,[],2);
    
end



% Save full simulation data for specific K values (if eta=0)
if eta == 0
    if K == 1e-5 || K == 1e-3 || K == 0.1 || K == 1 
        save(['K' num2str(K) '_D_PD_x.dat'],'x')
        save(['K' num2str(K) '_D_PD_xc.dat'],'xcpos')
        save(['K' num2str(K) '_C_PD_x.dat'],'X')
        save(['K' num2str(K) '_A_PD_x.dat'],'XA')
        save(['K' num2str(K) '_D_PD_p.dat'],'p')
        save(['K' num2str(K) '_C_PD_p.dat'],'phat')
        save(['K' num2str(K) '_A_PD_p.dat'],'pA')
        save(['K' num2str(K) '_D_PD_lambda.dat'],'lambda')
        save(['K' num2str(K) '_C_PD_lambda.dat'],'lambdahat')
        save(['K' num2str(K) '_A_PD_lambda.dat'],'lambdaA')
        save(['K' num2str(K) '_D_PD_t.dat'],'t')
        save(['K' num2str(K) '_C_PD_t.dat'],'T')
        save(['K' num2str(K) '_A_PD_t.dat'],'TA')
    end
end

% Save full simulation data for specific eta values (if K=0)
if K == 0
    if eta == 1e-5 || eta == 1e-3 || eta == 0.1 || eta == 1
        save(['eta' num2str(eta) '_D_PD_x.dat'],'x')
        save(['eta' num2str(eta) '_D_PD_xc.dat'],'xcpos')
        save(['eta' num2str(eta) '_C_PD_x.dat'],'X')
        save(['eta' num2str(eta) '_A_PD_x.dat'],'XA')
        save(['eta' num2str(eta) '_D_PD_p.dat'],'p')
        save(['eta' num2str(eta) '_C_PD_p.dat'],'phat')
        save(['eta' num2str(eta) '_A_PD_p.dat'],'pA')
        save(['eta' num2str(eta) '_D_PD_lambda.dat'],'lambda')
        save(['eta' num2str(eta) '_C_PD_lambda.dat'],'lambdahat')
        save(['eta' num2str(eta) '_A_PD_lambda.dat'],'lambdaA')
        save(['eta' num2str(eta) '_D_PD_t.dat'],'t')
        save(['eta' num2str(eta) '_C_PD_t.dat'],'T')
        save(['eta' num2str(eta) '_A_PD_t.dat'],'TA')
    end
end

% keep track of how many simulations have been completed & saved
disp(['Sim ' num2str(loop) ' of ' num2str(param) 'completed'])

end

% Save Parameter Data for K values (eta=0)
if eta == 0
    save('K_PD_Ks.dat','Ks')
    save('K_D_PD_MaxP.dat','maxp')
    save('K_C_PD_MaxP.dat','maxphat')
    save('K_A_PD_MaxP.dat','maxpA')
    save('K_D_PD_MinP.dat','minp')
    save('K_C_PD_MinP.dat','minphat')
    save('K_A_PD_MinP.dat','minpA')
    save('K_D_PD_MinLambda.dat','minstretch')
    save('K_C_PD_MinLambda.dat','minstretchhat')
    save('K_A_PD_MinLambda.dat','minstretchA')
    save('K_D_PD_L.dat','L')
    save('K_C_PD_L.dat','Lhat')
    save('K_A_PD_L.dat','LA')
end
% Save Parameter Data for eta values (K=0)
if K == 0
    save('eta_PD_etas.dat','etas')
    save('eta_D_PD_MaxP.dat','maxp')
    save('eta_C_PD_MaxP.dat','maxphat')
    save('eta_A_PD_MaxP.dat','maxpA')
    save('eta_D_PD_MinP.dat','minp')
    save('eta_C_PD_MinP.dat','minphat')
    save('eta_A_PD_MinP.dat','minpA')
    save('eta_D_PD_MinLambda.dat','minstretch')
    save('eta_C_PD_MinLambda.dat','minstretchhat')
    save('eta_A_PD_MinLambda.dat','minstretchA')
    save('eta_D_PD_L.dat','L')
    save('eta_C_PD_L.dat','Lhat')
    save('eta_A_PD_L.dat','LA')
end


%% IN SCRIPT FUNCTIONS

%% Discrete ODE 

function dudt = NHBdry_ODE_PD_AW(t,u)

    if eta == 0
        PD = l0*(1-k1) + k1*l0*exp(-t);
        PDdt = -k1*l0*exp(-t);
        
    end
    if K == 0
        if t < 1
            PD = l0*(1-k1*(3*t^2-2*t^3));
            PDdt = 6*l0*k1*(t^2-t);
        else
            PD = l0*(1-k1);
            PDdt = 0;
        end
    end
    
    
% % Cell stretches
stretch = (u(2:N+1)-u(1:N))'./L0;
% Cell width (incompressibility constraint)
Width = gam0./stretch; 

% % Initialise Force Vector
f = zeros(1,N); 
% Shared boundary width
Wforce=mean([Width(2:N); Width(1:N-1)]);
% Force balance at boundaries
f(2:N) = Wforce.*...
    (mu(2:N).*(stretch(2:N).^2 - stretch(2:N).^(-2))...
    - mu(1:N-1).*(stretch(1:N-1).^2 - stretch(1:N-1).^(-2)));
f(N+1) = PDdt;

% Mass Matrix for dx/dt
MD = eye(N+1,N+1);
for h=2:N
    MD(h,h-1) = -eta*Wforce(h-1)*(1 + stretch(h-1)^(-2));
    MD(h,h) = K + eta*...
        Wforce(h-1)*(2 + stretch(h-1)^(-2) + stretch(h)^(-2));
    MD(h,h+1) = -eta*Wforce(h-1)*(1 + stretch(h)^(-2));
end


% % Solve for dudt
dudt = MD\f';

end


%% Continuum PDE


function dUdT = NHBdry_contODE_PD_AW(T,U)

        if eta == 0
            PD = l0*(1-k1) + k1*l0*exp(-T);
            PDdt = -k1*l0*exp(-T);
            
        end
        if K == 0
            if T < 1
                PD = l0*(1-k1*(3*T^2-2*T^3));
                PDdt = 6*l0*k1*(T^2-T);
            else
                PD = l0*(1-k1);
                PDdt = 0;
            end
        end
        
% % Initialise derivative vector
forces = zeros(nc-1,1);

for itc1 = 2:nc-1
    
    % Discretisation scheme for PDE    
    forces(itc1) = gam0*(2*muhat(itc1))*((1/(dX*dX))...
        *(U(itc1+1)-2*U(itc1)+U(itc1-1)))...
        *(1 + ((1/(2*dX))*(U(itc1+1)-U(itc1-1)))^(-4)) + ...
       (1/(2*dX))*(muhat(itc1+1)-muhat(itc1-1))*...
        (1/(2*dX))*(U(itc1+1)-U(itc1-1))*...
        (1 - ((1/(2*dX))*(U(itc1+1)-U(itc1-1)))^(-4));
    
end
forces(nc) = PDdt;

% % Mass Matrix
MC = eye(nc,nc);
for h = 2:(nc-1)
    MC(h,h-1) = -eta*(1/dX/dX*(2*dX*(U(h+1)-U(h-1))^(-1) ...
        + (2*dX)^3*(U(h+1)-U(h-1))^(-3)) ...
        - 2*(2*dX)^4*(U(h+1)-U(h-1))^(-4)/dX/dX/(2*dX)*...
        (U(h+1)-2*U(h)+U(h-1)));
    MC(h,h) = K - eta*(-2/dX/dX)*(2*dX*(U(h+1)-U(h-1))^(-1) ...
        + (2*dX)^3*(U(h+1)-U(h-1))^(-3));
    MC(h,h+1) = -eta*(1/dX/dX*(2*dX*(U(h+1)-U(h-1))^(-1) ...
        + (2*dX)^3*(U(h+1)-U(h-1))^(-3)) ...
        + 2*(2*dX)^4*(U(h+1)-U(h-1))^(-4)/dX/dX/(2*dX)*...
        (U(h+1)-2*U(h)+U(h-1)));
end

% % Solve for dUdT
dUdT = MC\forces;

end


%% Discrete Pressure

function pfun = NHBdry_ODE_PD_AW_p(t,u)
    
    if eta == 0
        PDdt = -k1*l0*exp(-t);
    end
    if K == 0
        if t < 1
            PDdt = -l0*k1*(6*t-6*t^2);
        else
            PDdt =  0;
        end
    end
    
    
% % Cell stretches
stretch = (u(2:N+1)-u(1:N))/L0;
% Cell width (incompressibility constraint)
Width = gam0./stretch; 
% % Initialise Force Vector
f = zeros(1,N); 
% Shared boundary width
Wforce=mean([Width(2:N); Width(1:N-1)]);
% Force balance at boundaries
f(2:N) = Wforce.*...
    (mu(2:N).*(stretch(2:N).^2 - stretch(2:N).^(-2))...
    - mu(1:N-1).*(stretch(1:N-1).^2 - stretch(1:N-1).^(-2)));
f(N+1) = PDdt;


% % Mass Matrix
MD = eye(N+1,N+1);
for h=2:N
    MD(h,h-1) = -eta*Wforce(h-1)*(1 + stretch(h-1)^(-2));
    MD(h,h) = K + eta*...
        Wforce(h-1)*(2 + stretch(h-1)^(-2) + stretch(h)^(-2));
    MD(h,h+1) = -eta*Wforce(h-1)*(1 + stretch(h)^(-2));
end

% % Solve for dudt
dudt = MD\f';

pfun = mu(1:N).*stretch(1:N).^(-2)...
    -2*eta*stretch(1:N).^(-2).*(dudt(2:N+1)' - dudt(1:N)');

end


%% Continuum Pressure

function phatfun = NHBdry_contODE_PD_AW_phat(T,U)

    if eta == 0
        PD = l0*(1-k1) + k1*l0*exp(-T);
        PDdt = -k1*l0*exp(-T);
        
    end
    if K == 0
        if T < 1
            PD = l0*(1-k1*(3*T^2-2*T^3));
            PDdt = -l0*k1*(6*T-6*T^2);
        else
            PD = l0*(1-k1);
            PDdt = 0;
        end
    end


% % Initialise derivative vector
forces = zeros(1,nc);

for itc1 = 2:nc-1
    
    % Discretisation scheme for PDE    
    forces(itc1) = gam0*(2*muhat(itc1))*((1/(dX*dX))...
        *(U(itc1+1)-2*U(itc1)+U(itc1-1)))...
        *(1 + ((1/(2*dX))*(U(itc1+1)-U(itc1-1)))^(-4)) + ...
       (1/(2*dX))*(muhat(itc1+1)-muhat(itc1-1))*...
        (1/(2*dX))*(U(itc1+1)-U(itc1-1))*...
        (1 - ((1/(2*dX))*(U(itc1+1)-U(itc1-1)))^(-4));
    
end
forces(nc) = PDdt;

% % Mass Matrix
MC = eye(nc,nc);
for h = 2:(nc-1)
    MC(h,h-1) = -eta*(1/dX/dX*(2*dX*(U(h+1)-U(h-1))^(-1) ...
        + (2*dX)^3*(U(h+1)-U(h-1))^(-3)) ...
        - 2*(2*dX)^4*(U(h+1)-U(h-1))^(-4)/dX/dX/(2*dX)*...
        (U(h+1)-2*U(h)+U(h-1)));
    MC(h,h) = K - eta*(-2/dX/dX)*(2*dX*(U(h+1)-U(h-1))^(-1) ...
        + (2*dX)^3*(U(h+1)-U(h-1))^(-3));
    MC(h,h+1) = -eta*(1/dX/dX*(2*dX*(U(h+1)-U(h-1))^(-1) ...
        + (2*dX)^3*(U(h+1)-U(h-1))^(-3)) ...
        + 2*(2*dX)^4*(U(h+1)-U(h-1))^(-4)/dX/dX/(2*dX)*...
        (U(h+1)-2*U(h)+U(h-1)));
end

dUdT = MC\forces';

phatfun = zeros(size(U'));
for itc11 = 2:nc-1
    phatfun(itc11) = muhat(itc11).*((1/(2*dX))*...
        (U(itc11+1)-U(itc11-1))).^(-2)...
       -2*eta*((1/(2*dX))*(U(itc11+1)-U(itc11-1))).^(-2)...
      .*((1/(2*dX))*(dUdT(itc11+1)-dUdT(itc11-1)));
end
phatfun(1) = muhat(1)*((1/2/dX)*(-3*U(1)+4*U(2)-U(3)))^(-2)...
    -2*eta*((1/2/dX)*(-3*U(1)+4*U(2)-U(3)))^(-2)...
    *((1/2/dX)*(-3*dUdT(1)+4*dUdT(2)-dUdT(3)));
phatfun(nc) = muhat(nc)*...
    ((1/2/dX)*(3*U(nc)-4*U(nc-1)+U(nc-2)))^(-2)...
    -2*eta*((1/2/dX)*(3*U(nc)-4*U(nc-1)+U(nc-2)))^(-2)...
    *((1/2/dX)*(3*dUdT(nc)-4*dUdT(nc-1)+dUdT(nc-2)));

end

end